package com.marvinelsen.willow.ui.cells

import com.marvinelsen.willow.config.Config
import com.marvinelsen.willow.config.Script
import com.marvinelsen.willow.domain.entities.Sentence
import com.marvinelsen.willow.ui.util.createContextMenuForSentence
import javafx.application.HostServices
import javafx.beans.binding.Bindings
import javafx.scene.control.Label
import javafx.scene.control.ListCell
import javafx.scene.control.ListView
import javafx.scene.layout.VBox
import javafx.util.Callback
import java.util.ResourceBundle

class SentenceCellFactory(
    private val resources: ResourceBundle,
    private val config: Config,
    private val hostServices: HostServices,
) : Callback<ListView<Sentence?>, ListCell<Sentence?>> {

    override fun call(listView: ListView<Sentence?>): ListCell<Sentence?> {
        val sentenceCell = SentenceCell(resources, config, hostServices)
        sentenceCell.prefWidthProperty().bind(listView.widthProperty().subtract(CELL_PADDING))
        return sentenceCell
    }

    companion object {
        private const val CELL_PADDING = 16
    }
}

private class SentenceCell(
    private val resources: ResourceBundle,
    private val config: Config,
    private val hostServices: HostServices,
) : ListCell<Sentence?>() {

    private val labelSentence by lazy {
        Label().apply {
            styleClass.add("sentence")
            isWrapText = true
        }
    }

    private val root by lazy {
        VBox(labelSentence)
    }

    init {
        text = null
    }

    override fun updateItem(sentence: Sentence?, empty: Boolean) {
        super.updateItem(sentence, empty)
        if (empty || sentence == null) {
            graphic = null
            contextMenu = null
        } else {
            labelSentence.textProperty().bind(
                Bindings.createStringBinding(
                    {
                        when (config.script.value!!) {
                            Script.SIMPLIFIED -> sentence.simplified
                            Script.TRADITIONAL -> sentence.traditional
                        }
                    },
                    config.script
                )
            )

            contextMenu = createContextMenuForSentence(sentence, resources, hostServices)
            graphic = root
        }
    }
}
