package com.marvinelsen.willow.ui.controllers

import com.marvinelsen.willow.Model
import com.marvinelsen.willow.domain.SearchMode
import javafx.fxml.FXML
import javafx.scene.control.Button
import javafx.scene.control.TextField
import javafx.scene.control.ToggleGroup

class SearchController(private val model: Model) {
    @FXML
    private lateinit var buttonUndo: Button

    @FXML
    private lateinit var buttonRedo: Button

    @FXML
    private lateinit var searchModeToggleGroup: ToggleGroup

    @FXML
    private lateinit var textFieldSearch: TextField

    @FXML
    @Suppress("UnusedPrivateMember")
    private fun initialize() {
        textFieldSearch.textProperty().addListener { _, _, _ -> search() }
        searchModeToggleGroup.selectedToggleProperty().addListener { _, _, _ -> search() }

        buttonUndo.disableProperty().bind(model.canUndo.not())
        buttonRedo.disableProperty().bind(model.canRedo.not())
    }

    private fun search() {
        val searchQuery = textFieldSearch.text
        val searchMode = searchModeToggleGroup.selectedToggle.userData as SearchMode

        if (searchQuery.isNullOrBlank()) {
            return
        }

        model.search(searchQuery, searchMode)
    }

    fun onButtonRedoAction() {
        model.redoSelection()
    }

    fun onButtonUndoAction() {
        model.undoSelection()
    }
}
