package com.marvinelsen.willow.ui.controllers.dialogs

import com.marvinelsen.willow.config.Config
import com.marvinelsen.willow.config.Pronunciation
import com.marvinelsen.willow.config.Script
import com.marvinelsen.willow.config.Theme
import com.marvinelsen.willow.ui.formatters.FontSizeTextFormatter
import javafx.fxml.FXML
import javafx.scene.control.CheckBox
import javafx.scene.control.ComboBox
import javafx.scene.control.Spinner
import java.util.Locale

@Suppress("UnusedPrivateMember")
class PreferencesDialogController(private val config: Config) {
    @FXML
    private lateinit var comboBoxLocale: ComboBox<Locale>

    @FXML
    private lateinit var comboBoxTheme: ComboBox<Theme>

    @FXML
    private lateinit var comboBoxScript: ComboBox<Script>

    @FXML
    private lateinit var comboBoxPronunciationSearchResults: ComboBox<Pronunciation>

    @FXML
    private lateinit var comboBoxPronunciationDetails: ComboBox<Pronunciation>

    @FXML
    private lateinit var checkBoxShowPronunciationSearchResults: CheckBox

    @FXML
    private lateinit var checkBoxShowDefinitionSearchResults: CheckBox

    @FXML
    private lateinit var spinnerHeadwordFontSizeDetails: Spinner<Int>

    @FXML
    private lateinit var spinnerPronunciationFontSizeDetails: Spinner<Int>

    @FXML
    private lateinit var spinnerHeadwordFontSizeSearchResults: Spinner<Int>

    @FXML
    private lateinit var spinnerPronunciationFontSizeSearchResults: Spinner<Int>

    @FXML
    private lateinit var spinnerDefinitionFontSizeSearchResults: Spinner<Int>

    @FXML
    private fun initialize() {
        comboBoxLocale.items.addAll(
            setOf(
                Locale.ENGLISH,
                Locale.GERMAN,
                Locale.CHINA,
                Locale.TAIWAN
            )
        )

        comboBoxTheme.valueProperty().bindBidirectional(config.theme)
        comboBoxScript.valueProperty().bindBidirectional(config.script)
        comboBoxLocale.valueProperty().bindBidirectional(config.locale)
        comboBoxPronunciationSearchResults.valueProperty().bindBidirectional(config.searchResults.pronunciation)
        comboBoxPronunciationDetails.valueProperty().bindBidirectional(config.details.pronunciation)

        checkBoxShowDefinitionSearchResults
            .selectedProperty()
            .bindBidirectional(config.searchResults.shouldShowDefinition)
        checkBoxShowPronunciationSearchResults
            .selectedProperty()
            .bindBidirectional(config.searchResults.shouldShowPronunciation)

        with(spinnerHeadwordFontSizeDetails) {
            editor.textFormatter = FontSizeTextFormatter()
            valueFactory.value = config.details.headwordFontSize.value
            valueProperty().addListener { _, _, newValue ->
                config.details.headwordFontSize.value = newValue
            }
        }

        with(spinnerPronunciationFontSizeDetails) {
            editor.textFormatter = FontSizeTextFormatter()
            valueFactory.value = config.details.pronunciationFontSize.value
            valueProperty().addListener { _, _, newValue ->
                config.details.pronunciationFontSize.value = newValue
            }
        }

        with(spinnerHeadwordFontSizeSearchResults) {
            editor.textFormatter = FontSizeTextFormatter()
            valueFactory.value = config.searchResults.headwordFontSize.value
            valueProperty().addListener { _, _, newValue ->
                config.searchResults.headwordFontSize.value = newValue
            }
        }

        with(spinnerPronunciationFontSizeSearchResults) {
            editor.textFormatter = FontSizeTextFormatter()
            valueFactory.value = config.searchResults.pronunciationFontSize.value
            valueProperty().addListener { _, _, newValue ->
                config.searchResults.pronunciationFontSize.value = newValue
            }
        }

        with(spinnerDefinitionFontSizeSearchResults) {
            editor.textFormatter = FontSizeTextFormatter()
            valueFactory.value = config.searchResults.definitionFontSize.value
            valueProperty().addListener { _, _, newValue ->
                config.searchResults.definitionFontSize.value = newValue
            }
        }
    }
}
