package com.marvinelsen.willow.ui.util

import com.marvinelsen.willow.domain.entities.DictionaryEntry
import com.marvinelsen.willow.domain.entities.Sentence
import javafx.application.HostServices
import javafx.event.EventHandler
import javafx.scene.control.ContextMenu
import javafx.scene.control.MenuItem
import java.net.URLEncoder
import java.nio.charset.StandardCharsets
import java.util.ResourceBundle

fun createContextMenuForEntry(
    entry: DictionaryEntry,
    resourceBundle: ResourceBundle,
    hostServices: HostServices,
) = ContextMenu().apply {
    val menuItemCopyHeadword =
        MenuItem(resourceBundle.getString("menubar.edit.copy.headword")).apply {
            onAction = EventHandler { ClipboardHelper.copyString(entry.traditional) }
        }

    val menuItemCopyPronunciation =
        MenuItem(resourceBundle.getString("menubar.edit.copy.pronunciation")).apply {
            onAction = EventHandler { ClipboardHelper.copyString(entry.pinyinWithToneMarks) }
        }

    val menuItemSearchOnWeb =
        MenuItem(resourceBundle.getString("menubar.edit.search-web")).apply {
            onAction = EventHandler {
                val query = URLEncoder.encode(entry.traditional, StandardCharsets.UTF_8)
                hostServices.showDocument("https://duckduckgo.com/?q=$query")
            }
        }

    items.addAll(menuItemCopyHeadword, menuItemCopyPronunciation, menuItemSearchOnWeb)
}

fun createContextMenuForSentence(
    sentence: Sentence,
    resourceBundle: ResourceBundle,
    hostServices: HostServices,
) = ContextMenu().apply {
    val menuItemCopySentence =
        MenuItem(resourceBundle.getString("menubar.edit.copy.sentence")).apply {
            onAction = EventHandler { ClipboardHelper.copyString(sentence.traditional) }
        }

    val menuItemSearchOnWeb =
        MenuItem(resourceBundle.getString("menubar.edit.search-web")).apply {
            onAction = EventHandler {
                val query = URLEncoder.encode(sentence.traditional, StandardCharsets.UTF_8)
                hostServices.showDocument("https://duckduckgo.com/?q=$query")
            }
        }

    items.addAll(menuItemCopySentence, menuItemSearchOnWeb)
}
